//  file: ODRBC.CPP
//
//  TODRadioButton implementation
//      Defines the basic behavior for owner-drawn radio buttons.
//
// This source code was written by Colin Robertson and, to
// the extent I can claim ownership rights herein, is dedicated to
// the public domain. Use as you see fit. I also disclaim any
// warranty or liability for this code or anything derived from
// it.
//
// Colin Robertson can be reached at:
// CIS: 70550,3306
// Internet: colinr@dsinet.uucp
//

#include "odrbc.h"

//----------------------------------------------------------------------
//  TODRadioButton Destructor:
//
//  Remove all of the bitmap images we loaded from the resource file.
//

TODRadioButton::~TODRadioButton()
    {
    DeleteObject( hBtnUp );
    DeleteObject( hBtnDn );
    DeleteObject( hBtnUpF );
    DeleteObject( hBtnDnF );
    DeleteObject( hBtnGray );
    }


//---------------------------------------------------------------------
//  Init:
//
//  Load the set of bitmaps for the radio button from the resource file.
//  The numbering scheme works as follows: use a resource ID less than
//  1000 for the radio button. Create bitmaps (preferably of the same
//  dimensions, in this implementation) for each of the five possible
//  states of the button ( up without focus; depressed without focus;
//  up with focus; down with focus; and greyed out ). Number the
//  resource IDs for these bitmaps as 1000 + ID, 3000 + ID, 5000 + ID,
//  7000 + ID, and 9000 + ID, respectively. ( I figured even thousands
//  might come in handy if I ever wanted an EGA version of the control.)
//  This routine will pick up all of the bitmaps for later drawing and
//  save off the dimensions.
//

void TODRadioButton::Init( int nId )
    {
    // load resources
    HINSTANCE hInstance = GetApplication()->hInstance;
    hBtnUp = LoadBitmap( hInstance, MAKEINTRESOURCE( UpNoFocus + nId ));
    hBtnDn = LoadBitmap( hInstance, MAKEINTRESOURCE( DnNoFocus + nId ));
    hBtnUpF = LoadBitmap( hInstance, MAKEINTRESOURCE( UpFocus + nId ));
    hBtnDnF = LoadBitmap( hInstance, MAKEINTRESOURCE( DnFocus + nId ));
    hBtnGray = LoadBitmap( hInstance, MAKEINTRESOURCE( Grayed + nId ));

    // store dimensions of bitmap, plus 2-pixel wide border
    BITMAP bitMap;
    GetObject( hBtnUp, sizeof( bitMap ), (LPSTR)&bitMap );
    m_W = bitMap.bmWidth + 2;
    m_H = bitMap.bmHeight + 2;
    }


//--------------------------------------------------------------------------
//  WMDrawItem:
//
//  This is the routine that actually draws the button in its various states.
//  Sometimes this routine gets called more often than I expected; Sorry, I
//  haven't tracked down why yet. It may cause some flicker.
//

void TODRadioButton::WMDrawItem( RTMessage Msg )
    {
    LPDRAWITEMSTRUCT lpdis = (LPDRAWITEMSTRUCT)Msg.LParam;
    HBITMAP hbmp;

    // select the bitmap to draw
    switch ( GetCheck() )
        {
    case BF_CHECKED:
        hbmp = (( lpdis->itemState & ODS_FOCUS ) ? hBtnDnF : hBtnDn );
        break;
    case BF_UNCHECKED:
        hbmp = (( lpdis->itemState & ODS_FOCUS ) ? hBtnUpF : hBtnUp );
        break;
    default:
        hbmp = hBtnGray;
        break;
        }

    // draw black border around button if selected, ltgray otherwise
    RECT rc;
    SetRect( &rc, lpdis->rcItem.left, lpdis->rcItem.top,
             lpdis->rcItem.left + m_W,
             lpdis->rcItem.top + m_H );
    FillRect( lpdis->hDC, &rc, (HBRUSH)GetStockObject(
                ( lpdis->itemState & ODS_SELECTED ) ?
                  BLACK_BRUSH : LTGRAY_BRUSH ));

    // draw the bitmap
    HDC hDCmem = CreateCompatibleDC( lpdis->hDC );
    SelectObject( hDCmem, hbmp );
    BitBlt( lpdis->hDC, lpdis->rcItem.left + 1, lpdis->rcItem.top + 1,
            m_W - 1, m_H - 1, hDCmem, 0, 0, SRCCOPY );
    DeleteDC( hDCmem );
    }

